@@ -211,6 +211,9 @@ void Render()
 	if (g_Game && g_Game->IsGameStarted())
 		g_Game->GetView()->BeginFrame();
 
+	if (g_Game)
+		g_Renderer.SetSimulation(g_Game->GetSimulation2());
+
 	// start new frame
 	g_Renderer.BeginFrame();
 
@@ -571,7 +574,15 @@ static void InitRenderer()
 	// set renderer options from command line options - NOVBO must be set before opening the renderer
 	g_Renderer.SetOptionBool(CRenderer::OPT_NOVBO, g_NoGLVBO);
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHADOWS, g_Shadows);
-	g_Renderer.SetOptionBool(CRenderer::OPT_FANCYWATER, g_FancyWater);
+
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATERNORMAL, g_WaterNormal);
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATERREALDEPTH, g_WaterRealDepth);
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATERFOAM, g_WaterFoam);
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATERCOASTALWAVES, g_WaterCoastalWaves);
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATERREFLECTION, g_WaterRefraction);
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATERREFRACTION, g_WaterReflection);
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATERSHADOW, g_WaterShadows);
+	
 	g_Renderer.SetRenderPath(CRenderer::GetRenderPathByName(g_RenderPath));
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHADOWPCF, g_ShadowPCF);
 	g_Renderer.SetOptionBool(CRenderer::OPT_PARTICLES, g_Particles);
@@ -983,7 +994,15 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 		if (!Autostart(args))
 		{
 			const bool setup_gui = ((flags & INIT_NO_GUI) == 0);
-			InitPs(setup_gui, L"page_pregame.xml", JSVAL_VOID);
+			// We only want to display the splash screen at startup
+			CScriptValRooted data;
+			if (g_GUI)
+			{
+				ScriptInterface& scriptInterface = g_GUI->GetScriptInterface();
+				scriptInterface.Eval("({})", data);
+				scriptInterface.SetProperty(data.get(), "isStartup", true);
+			}
+			InitPs(setup_gui, L"page_pregame.xml", data.get());
 		}
 	}
 	catch (PSERROR_Game_World_MapLoadFailed e)
