@@ -37,6 +37,8 @@
 #include "renderer/ModelVertexRenderer.h"
 #include "renderer/Renderer.h"
 #include "renderer/RenderModifiers.h"
+#include "renderer/SkyManager.h"
+#include "renderer/WaterManager.h"
 
 #include <boost/weak_ptr.hpp>
 
@@ -147,9 +149,9 @@ void ModelRenderer::BuildColor4ub(
 }
 
 
-void ModelRenderer::GenTangents(const CModelDefPtr& mdef, std::vector<float>& newVertices)
+void ModelRenderer::GenTangents(const CModelDefPtr& mdef, std::vector<float>& newVertices, bool gpuSkinning)
 {
-	MikkTSpace ms(mdef, newVertices);
+	MikkTSpace ms(mdef, newVertices, gpuSkinning);
 
 	ms.generate();
 }
@@ -705,6 +707,22 @@ void ShaderModelRenderer::Render(const RenderModifierPtr& modifier, const CShade
 									shader->Uniform(binding, time, 0,0,0);
 								}
 							}
+							else if (rq.first == RQUERY_WATER_TEX)
+							{
+								WaterManager* WaterMgr = g_Renderer.GetWaterManager();
+								double time = WaterMgr->m_WaterTexTimer;
+								double period = 1.6;
+								int curTex = (int)(time*60/period) % 60;
+								
+								if (WaterMgr->m_RenderWater && WaterMgr->WillRenderFancyWater())
+									shader->BindTexture("waterTex", WaterMgr->m_NormalMap[curTex]);
+								else
+									shader->BindTexture("waterTex", g_Renderer.GetTextureManager().GetErrorTexture());
+							}
+							else if (rq.first == RQUERY_SKY_CUBE)
+							{
+								shader->BindTexture("skyCube", g_Renderer.GetSkyManager()->GetSkyCube());
+							}
 						}
 
 						modifier->PrepareModel(shader, model);
