@@ -69,6 +69,7 @@
 #include "renderer/VertexBufferManager.h"
 #include "renderer/WaterManager.h"
 
+extern bool g_GameRestarted;
 
 ///////////////////////////////////////////////////////////////////////////////////
 // CRendererStatsTable - Profile display of rendering stats
@@ -276,6 +277,9 @@ public:
 
 	/// Shadow map
 	ShadowMap shadow;
+	
+	/// Postprocessing effect manager
+	CPostprocManager postprocManager;
 
 	/// Various model renderers
 	struct Models
@@ -425,7 +429,6 @@ CRenderer::CRenderer()
 
 	m_Options.m_NoVBO = false;
 	m_Options.m_RenderPath = RP_DEFAULT;
-	m_Options.m_FancyWater = false;
 	m_Options.m_Shadows = false;
 	m_Options.m_ShadowAlphaFix = true;
 	m_Options.m_ARBProgramShadow = true;
@@ -437,6 +440,7 @@ CRenderer::CRenderer()
 	m_Options.m_GPUSkinning = false;
 	m_Options.m_GenTangents = false;
 	m_Options.m_SmoothLOS = false;
+	m_Options.m_Postproc = false;
 	m_Options.m_ShowSky = false;
 
 	// TODO: be more consistent in use of the config system
@@ -445,6 +449,7 @@ CRenderer::CRenderer()
 	CFG_GET_USER_VAL("gpuskinning", Bool, m_Options.m_GPUSkinning);
 	CFG_GET_USER_VAL("gentangents", Bool, m_Options.m_GenTangents);
 	CFG_GET_USER_VAL("smoothlos", Bool, m_Options.m_SmoothLOS);
+	CFG_GET_USER_VAL("postproc", Bool, m_Options.m_Postproc);
 
 #if CONFIG2_GLES
 	// Override config option since GLES only supports GLSL
@@ -462,7 +467,15 @@ CRenderer::CRenderer()
 
 	m_Stats.Reset();
 	AddLocalProperty(L"particles", &m_Options.m_Particles, false);
-	AddLocalProperty(L"fancyWater", &m_Options.m_FancyWater, false);
+
+	AddLocalProperty(L"waternormal", &m_Options.m_WaterNormal, false);
+	AddLocalProperty(L"waterrealdepth", &m_Options.m_WaterRealDepth, false);
+	AddLocalProperty(L"waterreflection", &m_Options.m_WaterReflection, false);
+	AddLocalProperty(L"waterrefraction", &m_Options.m_WaterRefraction, false);
+	AddLocalProperty(L"waterfoam", &m_Options.m_WaterFoam, false);
+	AddLocalProperty(L"watercoastalwaves", &m_Options.m_WaterCoastalWaves, false);
+	AddLocalProperty(L"watershadow", &m_Options.m_WaterShadow, false);
+
 	AddLocalProperty(L"horizonHeight", &m->skyManager.m_HorizonHeight, false);
 	AddLocalProperty(L"waterMurkiness", &m->waterManager.m_Murkiness, false);
 	AddLocalProperty(L"waterReflTintStrength", &m->waterManager.m_ReflectionTintStrength, false);
@@ -579,7 +592,7 @@ void CRenderer::ReloadShaders()
 
 	if (GetRenderPath() == RP_SHADER && m_Options.m_GPUSkinning) // TODO: should check caps and GLSL etc too
 	{
-		m->Model.VertexGPUSkinningShader = ModelVertexRendererPtr(new InstancingModelRenderer(true, false));
+		m->Model.VertexGPUSkinningShader = ModelVertexRendererPtr(new InstancingModelRenderer(true, m_Options.m_GenTangents));
 		m->Model.NormalSkinned = ModelRendererPtr(new ShaderModelRenderer(m->Model.VertexGPUSkinningShader));
 		m->Model.TranspSkinned = ModelRendererPtr(new ShaderModelRenderer(m->Model.VertexGPUSkinningShader));
 	}
@@ -642,6 +655,9 @@ bool CRenderer::Open(int width, int height)
 	// Let component renderers perform one-time initialization after graphics capabilities and
 	// the shader path have been determined.
 	m->overlayRenderer.Initialize();
+	
+	if (m_Options.m_Postproc)
+		m->postprocManager.Initialize();
 
 	return true;
 }
@@ -654,6 +670,9 @@ void CRenderer::Resize(int width,int height)
 
 	m_Width = width;
 	m_Height = height;
+	
+	if (m_Options.m_Postproc)
+		m->postprocManager.RecreateBuffers();
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -668,8 +687,26 @@ void CRenderer::SetOptionBool(enum Option opt,bool value)
 			m_Options.m_Shadows = value;
 			MakeShadersDirty();
 			break;
-		case OPT_FANCYWATER:
-			m_Options.m_FancyWater = value;
+		case OPT_WATERNORMAL:
+			m_Options.m_WaterNormal = value;
+			break;
+		case OPT_WATERREALDEPTH:
+			m_Options.m_WaterRealDepth = value;
+			break;
+		case OPT_WATERFOAM:
+			m_Options.m_WaterFoam = value;
+			break;
+		case OPT_WATERCOASTALWAVES:
+			m_Options.m_WaterCoastalWaves = value;
+			break;
+		case OPT_WATERREFLECTION:
+			m_Options.m_WaterReflection = value;
+			break;
+		case OPT_WATERREFRACTION:
+			m_Options.m_WaterRefraction = value;
+			break;
+		case OPT_WATERSHADOW:
+			m_Options.m_WaterShadow = value;
 			break;
 		case OPT_SHADOWPCF:
 			m_Options.m_ShadowPCF = value;
@@ -699,8 +736,20 @@ bool CRenderer::GetOptionBool(enum Option opt) const
 			return m_Options.m_NoVBO;
 		case OPT_SHADOWS:
 			return m_Options.m_Shadows;
-		case OPT_FANCYWATER:
-			return m_Options.m_FancyWater;
+		case OPT_WATERNORMAL:
+			return m_Options.m_WaterNormal;
+		case OPT_WATERREALDEPTH:
+			return m_Options.m_WaterRealDepth;
+		case OPT_WATERFOAM:
+			return m_Options.m_WaterFoam;
+		case OPT_WATERCOASTALWAVES:
+			return m_Options.m_WaterCoastalWaves;
+		case OPT_WATERREFLECTION:
+			return m_Options.m_WaterReflection;
+		case OPT_WATERREFRACTION:
+			return m_Options.m_WaterRefraction;
+		case OPT_WATERSHADOW:
+			return m_Options.m_WaterShadow;
 		case OPT_SHADOWPCF:
 			return m_Options.m_ShadowPCF;
 		case OPT_PARTICLES:
@@ -795,12 +844,18 @@ void CRenderer::BeginFrame()
 
 	if (m->ShadersDirty)
 		ReloadShaders();
-
+	
 	m->Model.ModShader->SetShadowMap(&m->shadow);
 	m->Model.ModShader->SetLightEnv(m_LightEnv);
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
+void CRenderer::SetSimulation(CSimulation2* simulation)
+{
+	// set current simulation context for terrain renderer
+	m->terrainRenderer.SetSimulation(simulation);
+}
+
 // SetClearColor: set color used to clear screen in BeginFrame()
 void CRenderer::SetClearColor(SColor4ub color)
 {
@@ -818,7 +873,10 @@ void CRenderer::RenderShadowMap(const CShaderDefines& context)
 
 	{
 		PROFILE("render patches");
+		glCullFace(GL_FRONT);
+		glEnable(GL_CULL_FACE);
 		m->terrainRenderer.RenderPatches();
+		glCullFace(GL_BACK);
 	}
 
 	CShaderDefines contextCast = context;
@@ -1370,6 +1428,9 @@ void CRenderer::RenderSubmissions()
 	PROFILE3("render submissions");
 	
 	GetScene().GetLOSTexture().InterpolateLOS();
+	
+	if (m_Options.m_Postproc)
+		m->postprocManager.CaptureRenderOutput();
 
 	CShaderDefines context = m->globalContext;
 
@@ -1414,15 +1475,33 @@ void CRenderer::RenderSubmissions()
 		waterScissor = m->terrainRenderer.ScissorWater(m_ViewCamera.GetViewProjection());
 		if (waterScissor.GetVolume() > 0 && m_WaterManager->WillRenderFancyWater())
 		{
-			SScreenRect reflectionScissor = RenderReflections(context, waterScissor);
-			SScreenRect refractionScissor = RenderRefractions(context, waterScissor);
-
 			PROFILE3_GPU("water scissor");
 			SScreenRect dirty;
-			dirty.x1 = std::min(reflectionScissor.x1, refractionScissor.x1);
-			dirty.y1 = std::min(reflectionScissor.y1, refractionScissor.y1);
-			dirty.x2 = std::max(reflectionScissor.x2, refractionScissor.x2);
-			dirty.y2 = std::max(reflectionScissor.y2, refractionScissor.y2);
+			if (m_Options.m_WaterRefraction && m_Options.m_WaterReflection)
+			{
+				SScreenRect reflectionScissor = RenderReflections(context, waterScissor);
+				SScreenRect refractionScissor = RenderRefractions(context, waterScissor);
+				dirty.x1 = std::min(reflectionScissor.x1, refractionScissor.x1);
+				dirty.y1 = std::min(reflectionScissor.y1, refractionScissor.y1);
+				dirty.x2 = std::max(reflectionScissor.x2, refractionScissor.x2);
+				dirty.y2 = std::max(reflectionScissor.y2, refractionScissor.y2);
+ 			}
+			else if (m_Options.m_WaterRefraction)
+			{
+				SScreenRect refractionScissor = RenderRefractions(context, waterScissor);
+				dirty.x1 = refractionScissor.x1;
+				dirty.y1 = refractionScissor.y1;
+				dirty.x2 = refractionScissor.x2;
+				dirty.y2 = refractionScissor.y2;
+			}
+			else
+			{
+				SScreenRect reflectionScissor = RenderReflections(context, waterScissor);
+				dirty.x1 = reflectionScissor.x1;
+				dirty.y1 = reflectionScissor.y1;
+				dirty.x2 = reflectionScissor.x2;
+				dirty.y2 = reflectionScissor.y2;
+			}
 			if (dirty.x1 < dirty.x2 && dirty.y1 < dirty.y2)
 			{
 				glEnable(GL_SCISSOR_TEST);
@@ -1461,7 +1540,7 @@ void CRenderer::RenderSubmissions()
 		RenderTransparentModels(context, TRANSPARENT_OPAQUE);
 		ogl_WarnIfError();
 
-		m->terrainRenderer.RenderWater();
+		m->terrainRenderer.RenderWater(context, &m->shadow);
 		ogl_WarnIfError();
 
 		// render transparent stuff again, but only the blended parts that overlap water
@@ -1489,6 +1568,12 @@ void CRenderer::RenderSubmissions()
 		RenderParticles();
 		ogl_WarnIfError();
 	}
+	
+	if (m_Options.m_Postproc)
+	{
+		m->postprocManager.ApplyPostproc();
+		m->postprocManager.ReleaseRenderOutput();
+	}
 
 	if (m_Options.m_Silhouettes)
 	{
@@ -1513,6 +1598,7 @@ void CRenderer::RenderSubmissions()
 	// render overlays that should appear on top of all other objects
 	m->overlayRenderer.RenderForegroundOverlays(m_ViewCamera);
 	ogl_WarnIfError();
+	
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
@@ -2029,3 +2115,8 @@ CMaterialManager& CRenderer::GetMaterialManager()
 {
 	return m->materialManager;
 }
+
+CPostprocManager& CRenderer::GetPostprocManager()
+{
+	return m->postprocManager;
+}
\ No newline at end of file
