@@ -83,6 +83,9 @@ struct ShadowMapInternals
 	// Copy of renderer's standard view camera, saved between
 	// BeginRender and EndRender while we replace it with the shadow camera
 	CCamera SavedViewCamera;
+	
+	// Save the caller's FBO so it can be restored
+	GLint SavedViewFBO;
 
 	// Helper functions
 	void CalcShadowMatrices();
@@ -221,7 +224,6 @@ void ShadowMap::AddShadowedBound(const CBoundingBoxAligned& bounds)
 // projection and transformation matrices
 void ShadowMapInternals::CalcShadowMatrices()
 {
-	CRenderer& renderer = g_Renderer;
 	float minZ = ShadowBound[0].Z;
 
 	ShadowBound.IntersectFrustumConservative(LightspaceCamera.GetFrustum());
@@ -273,7 +275,7 @@ void ShadowMapInternals::CalcShadowMatrices()
 	LightProjection._22 = scale.Y;
 	LightProjection._24 = (shift.Y + offsetY) * scale.Y;
 	LightProjection._33 = scale.Z;
-	LightProjection._34 = shift.Z * scale.Z + renderer.m_ShadowZBias;
+	LightProjection._34 = shift.Z * scale.Z;
 	LightProjection._44 = 1.0;
 
 	// Calculate texture matrix by creating the clip space to texture coordinate matrix
@@ -318,6 +320,9 @@ void ShadowMapInternals::CreateTexture()
 		pglDeleteFramebuffersEXT(1, &Framebuffer);
 		Framebuffer = 0;
 	}
+	
+	// save the caller's FBO	
+	glGetIntegerv(GL_FRAMEBUFFER_BINDING, &SavedViewFBO);
 
 	pglGenFramebuffersEXT(1, &Framebuffer);
 
@@ -433,7 +438,7 @@ void ShadowMapInternals::CreateTexture()
 
 	GLenum status = pglCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 
-	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, SavedViewFBO);
 
 	if (status != GL_FRAMEBUFFER_COMPLETE_EXT)
 	{
@@ -450,6 +455,9 @@ void ShadowMapInternals::CreateTexture()
 void ShadowMap::BeginRender()
 {
 	// HACK HACK: this depends in non-obvious ways on the behaviour of the caller
+	
+	// save caller's FBO
+	glGetIntegerv(GL_FRAMEBUFFER_BINDING, &m->SavedViewFBO);
 
 	// Calc remaining shadow matrices
 	m->CalcShadowMatrices();
@@ -502,7 +510,7 @@ void ShadowMap::EndRender()
 
 	{
 		PROFILE("unbind framebuffer");
-		pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+		pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m->SavedViewFBO);
 	}
 
 	glViewport(0, 0, g_Renderer.GetWidth(), g_Renderer.GetHeight());
