@@ -33,6 +33,7 @@
 #include "lib/file/vfs/vfs_util.h"
 #include "maths/MathUtil.h"
 #include "ps/CLogger.h"
+#include "ps/ConfigDB.h"
 #include "ps/Filesystem.h"
 #include "ps/Loader.h"
 #include "ps/Profile.h"
@@ -68,8 +69,12 @@ public:
 
 		RegisterFileReloadFunc(ReloadChangedFileCB, this);
 
-// 		m_EnableOOSLog = true; // TODO: this should be a command-line flag or similar
-// 		m_EnableSerializationTest = true; // TODO: this should too
+		// Tests won't have config initialised
+		if (CConfigDB::IsInitialised())
+		{
+			CFG_GET_USER_VAL("ooslog", Bool, m_EnableOOSLog);
+			CFG_GET_USER_VAL("serializationtest", Bool, m_EnableSerializationTest);
+		}
 	}
 
 	~CSimulation2Impl()
@@ -385,7 +390,7 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 
 			VfsPath mapfilename(VfsPath("maps/scenarios") / (mapFile + L".pmp"));
 			mapReader->LoadMap(mapfilename, &secondaryTerrain, NULL, NULL, NULL, NULL, NULL, NULL,
-				NULL, &secondaryContext, INVALID_PLAYER, true); // throws exception on failure
+				NULL, NULL, &secondaryContext, INVALID_PLAYER, true); // throws exception on failure
 		}
 		else
 		{
