@@ -46,8 +46,6 @@ CSoundBase::~CSoundBase()
 		CSoundData::ReleaseSoundData(m_SoundData);
 		m_SoundData = 0;
 	}
-	if (m_Name)
-		delete m_Name;
 }
 
 void CSoundBase::ResetVars()
@@ -62,7 +60,6 @@ void CSoundBase::ResetVars()
 	m_EndVolume = 0;
 
 	ResetFade();
-	m_Name = new std::string("sound name");
 }
 
 void CSoundBase::ResetFade()
@@ -78,15 +75,24 @@ void CSoundBase::SetGain(ALfloat gain)
 {
 	AL_CHECK
 
-	alSourcef(m_ALSource, AL_GAIN, gain);
-
-	AL_CHECK
+	if ( m_ALSource )	
+	{
+		alSourcef(m_ALSource, AL_GAIN, gain);
+		AL_CHECK
+	}
 }
 
 void CSoundBase::SetRollOff(ALfloat rolls)
 {
-   alSourcef(m_ALSource, AL_ROLLOFF_FACTOR, rolls);
-	AL_CHECK
+	if ( m_ALSource )	
+   	{
+		alSourcef(m_ALSource, AL_REFERENCE_DISTANCE, 70.0f);
+		AL_CHECK
+		alSourcef(m_ALSource, AL_MAX_DISTANCE, 200.0);
+		AL_CHECK
+   		alSourcef(m_ALSource, AL_ROLLOFF_FACTOR, rolls);
+		AL_CHECK
+	}
 }
 
 void CSoundBase::EnsurePlay()
@@ -97,57 +103,67 @@ void CSoundBase::EnsurePlay()
 
 void CSoundBase::SetCone(ALfloat innerCone, ALfloat outerCone, ALfloat coneGain)
 {
-	AL_CHECK	
-	alSourcef(m_ALSource, AL_CONE_INNER_ANGLE, innerCone);
-	AL_CHECK
-	alSourcef(m_ALSource, AL_CONE_OUTER_ANGLE, outerCone);
-	AL_CHECK
-	alSourcef(m_ALSource, AL_CONE_OUTER_GAIN, coneGain);
-	AL_CHECK
+	if ( m_ALSource )
+	{
+		AL_CHECK	
+		alSourcef(m_ALSource, AL_CONE_INNER_ANGLE, innerCone);
+		AL_CHECK
+		alSourcef(m_ALSource, AL_CONE_OUTER_ANGLE, outerCone);
+		AL_CHECK
+		alSourcef(m_ALSource, AL_CONE_OUTER_GAIN, coneGain);
+		AL_CHECK
+	}
 }
 
 void CSoundBase::SetPitch(ALfloat pitch)
 {
-	alSourcef(m_ALSource, AL_PITCH, pitch);
-	AL_CHECK
+	if ( m_ALSource )
+	{
+		alSourcef(m_ALSource, AL_PITCH, pitch);
+		AL_CHECK
+	}
 }
 
 void CSoundBase::SetDirection(const CVector3D& direction)
 {
-	alSourcefv(m_ALSource, AL_DIRECTION, direction.GetFloatArray());
-	AL_CHECK
+	if ( m_ALSource )
+	{
+		alSourcefv(m_ALSource, AL_DIRECTION, direction.GetFloatArray());
+		AL_CHECK
+	}
 }
 
 bool CSoundBase::InitOpenAL()
 {
 	alGetError(); /* clear error */
 	alGenSources(1, &m_ALSource);
-	long anErr = alGetError();
-
-	AL_CHECK
+	ALenum anErr = alGetError();
 
 	if (anErr == AL_NO_ERROR) 
-	{
-		ALfloat source0Pos[]={ -2.0, 0.0, 0.0};
-		ALfloat source0Vel[]={ 0.0, 0.0, 0.0};
-		
+	{		
 		alSourcef(m_ALSource,AL_PITCH,1.0f);
+		AL_CHECK
 		alSourcef(m_ALSource,AL_GAIN,1.0f);
-		alSourcefv(m_ALSource,AL_POSITION,source0Pos);
-		alSourcefv(m_ALSource,AL_VELOCITY,source0Vel);
+		AL_CHECK
 		alSourcei(m_ALSource,AL_LOOPING,AL_FALSE);
 		AL_CHECK
-
 		return true;
 	}
+	else
+	{
+		CSoundManager::al_ReportError( anErr, __func__, __LINE__);
+	}
 	return false;
 }
 
 bool CSoundBase::IsPlaying()
 {
-	int proc_state;
-	alGetSourceiv(m_ALSource, AL_SOURCE_STATE, &proc_state);
-	AL_CHECK
+	ALenum proc_state = AL_STOPPED;
+	if ( m_ALSource != 0 )
+	{
+		alGetSourceiv(m_ALSource, AL_SOURCE_STATE, &proc_state);
+		AL_CHECK
+	}
 
 	return (proc_state == AL_PLAYING);
 }
@@ -164,8 +180,11 @@ bool CSoundBase::IdleTask()
 
 void CSoundBase::SetLocation (const CVector3D& position)
 {
-	alSourcefv(m_ALSource,AL_POSITION, position.GetFloatArray());
-	AL_CHECK
+	if ( m_ALSource != 0 )
+	{
+		alSourcefv(m_ALSource,AL_POSITION, position.GetFloatArray());
+		AL_CHECK
+	}
 }
 
 bool CSoundBase::HandleFade()
@@ -182,10 +201,11 @@ bool CSoundBase::HandleFade()
 			Stop();
 		else if (curGain == m_EndVolume)
 		{
-			alSourcef(m_ALSource, AL_GAIN, curGain);	 
+			if (m_ALSource != 0)
+				alSourcef(m_ALSource, AL_GAIN, curGain);	 
 			ResetFade();
 		}
-		else
+		else if (m_ALSource != 0)
 			alSourcef(m_ALSource, AL_GAIN, curGain);	 
 
 		AL_CHECK
@@ -201,8 +221,11 @@ bool CSoundBase::GetLooping()
 void CSoundBase::SetLooping(bool loops)
 {
 	m_Looping = loops;
-	alSourcei(m_ALSource, AL_LOOPING, loops ? AL_TRUE : AL_FALSE);
-	AL_CHECK
+	if (m_ALSource != 0)
+	{
+		alSourcei(m_ALSource, AL_LOOPING, loops ? AL_TRUE : AL_FALSE);
+		AL_CHECK
+	}
 }
 
 void CSoundBase::Play()
@@ -243,16 +266,19 @@ void CSoundBase::PlayLoop()
 
 void CSoundBase::FadeToIn(ALfloat newVolume, double fadeDuration)
 {
-	int proc_state;
-	alGetSourceiv(m_ALSource, AL_SOURCE_STATE, &proc_state);
-	if (proc_state == AL_PLAYING)
-	{
-		m_StartFadeTime = timer_Time();
-		m_EndFadeTime = m_StartFadeTime + fadeDuration;
-		alGetSourcef(m_ALSource, AL_GAIN, &m_StartVolume);
-		m_EndVolume = newVolume;
+	if (m_ALSource != 0)
+	{		
+		ALenum proc_state;
+		alGetSourceiv(m_ALSource, AL_SOURCE_STATE, &proc_state);
+		if (proc_state == AL_PLAYING)
+		{
+			m_StartFadeTime = timer_Time();
+			m_EndFadeTime = m_StartFadeTime + fadeDuration;
+			alGetSourcef(m_ALSource, AL_GAIN, &m_StartVolume);
+			m_EndVolume = newVolume;
+		}
+		AL_CHECK
 	}
-	AL_CHECK
 }
 
 void CSoundBase::PlayAsMusic()
@@ -280,24 +306,12 @@ void CSoundBase::Stop()
 	}
 }
 
-const char* CSoundBase::Name()
+CStrW* CSoundBase::GetName()
 {
-	return m_Name->c_str();
-}
+	if ( m_SoundData )
+		return m_SoundData->GetFileName();
 
-std::string CSoundBase::GetName()
-{
-	return std::string(m_Name->c_str());
-}
-
-void CSoundBase::SetNameFromPath(char* fileLoc)
-{
-	std::string anst(fileLoc);
-	size_t pos = anst.find_last_of("/");
-	if(pos != std::wstring::npos)
-		m_Name->assign(anst.begin() + pos + 1, anst.end());
-	else
-		m_Name->assign(anst.begin(), anst.end());
+	return NULL;
 }
 
 #endif // CONFIG2_AUDIO
